#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <bass/bass.h>

#define rnd(n) (rand()%(n+1))

// --- BASS

HSAMPLE		music_sample;
HCHANNEL	music_channel;

HSAMPLE		sound_sample[32];
HCHANNEL	sound_channel[32];

void PlaySound(int i, int vol, int speed)
{
	if(sound_channel[i]==NULL)
		sound_channel[i]=BASS_SampleGetChannel(sound_sample[i], false);
	BASS_ChannelSetAttributes(sound_channel[i], 441*speed, vol, 0);
	BASS_ChannelPlay(sound_channel[i], true);
}

// --- BASS

struct Texture
{
	GLuint handle;
	int width;
	int height;
};

Texture textures[128];
Texture textures2[128];

struct Tile
{
	bool exists;
	int layer;
	int x;
	int y;
	int image;
	int size;
	int color;
	int timer;
	bool occupied;
};

struct Color
{
	float r, g, b;
};

Color colors[32];

#define MAX_TILES 1024

Tile tiles[MAX_TILES];

struct Plant
{
	int x;
	int y;
	int dir;
	int growth;
	int timer;
};

Plant plant;

struct Branch
{
	bool exists;
	int x;
	int y;
	int width1, width2;
	int growth1, growth2;
	int timer;
	int age;
	int color;
};

#define MAX_BRANCHES 128

Branch branches[MAX_BRANCHES];
int branch_interval;

struct Player
{
	int x;
	int y;
	int dx;
	int dy;
	int timer;
	int image;
	int dir;
	int air;
	bool jumping;
	int dropping;
	int pooning;
	int poon_x;
	int poon_y;
	int reload;
	int hurt;
	int score;
	int health;
};

Player player;

struct Particle
{
	bool exists;
	int type;
	int x;
	int y;
	int dx;
	int dy;
	int timer;
};

#define MAX_PARTICLES 128

Particle particles[MAX_PARTICLES];

struct Enemy
{
	bool exists;
	int type;
	int x;
	int y;
	int dx;
	int dy;
	int size;
	int hurt;
	int timer;
	int dir;
};

#define MAX_ENEMIES 64

Enemy enemies[MAX_ENEMIES];
int enemy_spawn;
int enemy_rate;

int scroll_x, scroll_y;
int scroll_offset;
int scroll_offset_delay;
int water_level;
int water_scroll;
int death_delay;

bool menu;
int menu_scroller;
char menu_text[1024];

bool escape=false;



Tile* GetNewTile()
{
	for(int i=0;i<MAX_TILES;i++)
		if(!tiles[i].exists)
		{
			tiles[i].exists=true;
			return &tiles[i];
		}
	return &tiles[MAX_TILES-1]; // Will mess things up, but hey... it won't reach this point, promise! ;)
}

void AddTile(int x, int y, int image, int layer, int size)
{
	for(int i=0;i<MAX_TILES;i++)
		if(!tiles[i].exists)
		{
			tiles[i].exists=true;
			tiles[i].layer=layer;
			tiles[i].x=x;
			tiles[i].y=y;
			tiles[i].image=image;
			tiles[i].size=size;
			tiles[i].color=0;
			tiles[i].timer=0;
			return;
		}
}

void ResetGame()
{
	for(int i=0;i<MAX_TILES;i++)
		tiles[i].exists=false;

	for(int i=0;i<MAX_BRANCHES;i++)
		branches[i].exists=false;

	for(int i=0;i<MAX_PARTICLES;i++)
		particles[i].exists=false;

	for(int i=0;i<MAX_ENEMIES;i++)
		enemies[i].exists=false;

	// Island
	AddTile(300-64, 300, 8, 0, 128+64);
	for(int i=0;i<5;i++)
	{
		if(i==0)
			AddTile(300-4*32+i*64, 300, 7, 0, 64);
		else if(i==4)
			AddTile(300-4*32+i*64, 300, 6, 0, 64);
		else
			AddTile(300-4*32+i*64, 300, 5, 0, 64);
	}
	
	plant.x=300;
	plant.y=300;
	plant.dir=0;
	plant.growth=8+rnd(4);
	plant.timer=64;

	scroll_x=0;
	scroll_y=0;
	scroll_offset=0;
	scroll_offset_delay=0;
	
	player.x=200;
	player.y=300-64;
	player.dx=0;
	player.dy=0;
	player.timer=0;
	player.dir=0;
	player.air=0;
	player.jumping=false;
	player.dropping=0;
	player.reload=0;
	player.pooning=0;
	player.hurt=0;
	player.health=4;
	player.score=0;

	enemy_spawn=400;
	enemy_rate=200;
	
	branch_interval=100;
	
	water_level=0;
	water_scroll=0;
	
	death_delay=0;
	
	menu=true;
	menu_scroller=-22*8;

	BASS_ChannelStop(music_channel);
}

void AddParticle(int type, int x, int y, int dx, int dy)
{
	for(int j=0;j<MAX_PARTICLES;j++)
		if(!particles[j].exists)
		{
			particles[j].exists=true;
			particles[j].type=type;
			particles[j].x=x;
			particles[j].y=y;
			particles[j].dx=dx;
			particles[j].dy=dy;
			particles[j].timer=0;
			break;
		}
}

bool Update()
{
	if(GetAsyncKeyState(VK_ESCAPE)&KF_UP)
	{
		if(!escape)
		{
			if(menu)
				return false;
			else
				ResetGame();
		}
		escape=true;
	}
	else
		escape=false;

	if(menu)
	{		
		menu_scroller++;
		if(menu_scroller>(int)strlen(menu_text)*8+8)
			menu_scroller=-22*8;

		if(GetAsyncKeyState(VK_RETURN)&KF_UP) // Start game
		{
			music_channel=BASS_SampleGetChannel(music_sample, false);
			BASS_ChannelSetAttributes(music_channel, 22050, 20, 0);
			BASS_ChannelSetFlags(music_channel, BASS_SAMPLE_LOOP);
			BASS_ChannelPlay(music_channel, true);
			menu=false;
		}
		else
			return true;
	}

	if(death_delay>0)
	{
		death_delay++;
		if(death_delay>150)
			ResetGame();
		return true;
	}

	// Update player	
	int mspeed=2;
	bool movement=false;
	if(GetAsyncKeyState(VK_RIGHT)&KF_UP)
	{
		player.dx+=mspeed;
		player.dir=0;
		movement=true;
	}
	if(GetAsyncKeyState(VK_LEFT)&KF_UP)
	{
		player.dx-=mspeed;
		player.dir=1;
		movement=true;
	}
	if((GetAsyncKeyState(VK_UP)&KF_UP) && (GetAsyncKeyState('X')&KF_UP)) // Launch harpoon
	{
		if(player.pooning==0)
		{
			PlaySound(9, 20, 100);
			player.pooning=1;
			player.poon_x=player.x+32;
			player.poon_y=player.y;
		}
	}
	if(!(GetAsyncKeyState(VK_UP)&KF_UP) && (GetAsyncKeyState('X')&KF_UP)) // Throw food
	{
		if(player.reload==0)
		{
			PlaySound(3, 20, 100);
			player.reload=1;
			if(player.dir==0)
				AddParticle(0, player.x+16, player.y+24, 5, -8*4);
			else
				AddParticle(0, player.x+16, player.y+24, -5, -8*4);
		}
	}
	else if(player.reload>10)
		player.reload=0;
	if(player.reload>0)
		player.reload++;
	if(GetAsyncKeyState('Z')&KF_UP) // Jump (or drop)
	{
		if(player.air==0 && !player.jumping)
		{
			if(GetAsyncKeyState(VK_DOWN)&KF_UP) // Drop, don't jump
			{
				player.jumping=true;
				player.air=1;
				player.dy=8;
				player.dropping=20; // Prevent landing on the branch we dropped from
			}
			else
			{
				PlaySound(0, 20, 150);
				player.jumping=true;
				player.air=1;
				player.dy=-32;
			}
		}
		else if(player.air>0 && player.air<20 && player.dy<0)
		{
			player.air++;
			player.dy-=8;
			if(player.dy<-64) player.dy=-64;
		}
	}
	else
	{
		if(player.jumping && player.air>0 && player.dy<-40)
		{
			player.y-=1;
			player.dy=-40;
			player.air=20;
		}
		player.jumping=false;
	}
	player.dy+=2;
	if(player.dy>128) player.dy=128;
	if(player.dx>32) player.dx=32;
	if(player.dx<-32) player.dx=-32;
	player.x+=player.dx/8;
	player.y+=player.dy/8;
	if(movement)
	{
		player.timer++;
		if(player.timer>=6*6)
			player.timer=0;
		player.image=31+player.timer/6;
	}
	else
	{
		player.timer=0;
		player.image=30;
		if(player.dx>0) player.dx-=2;
		if(player.dx<0) player.dx+=2;
		if(abs(player.dx)==1) player.dx=0;
	}
	if(player.air==0) // If on ground -> try falling
	{
		player.air=1;
		player.y+=1;
	}
	if(water_level<96*16 && player.dy>0 && player.x>300-6*32+16 && player.x<300+5*32+16 && player.y>=300-64) // Landed on island
	{
		player.air=0;
		player.y=300-64;
		player.dy=0;
	}
	if(player.dropping>0)
		player.dropping--;
	if(player.dy>0 && player.dropping==0 && player.pooning!=2) // Check for landing on branches
	{
		for(int i=0;i<MAX_BRANCHES;i++)
			if(branches[i].exists && player.x>branches[i].x-branches[i].width1*32-48
								  && player.x<branches[i].x+branches[i].width2*32+16
								  && player.y>branches[i].y-64
								  && player.y<branches[i].y-60+player.dy)
			{
				player.y=branches[i].y-64;
				player.air=0;
				if(player.dy>48) // Landed hard -> spawn leaves
				{
					PlaySound(1, 30, 150);
					for(int d=0;d<5;d++)
						AddParticle(6, player.x+rnd(32), branches[i].y, rnd(16)-8, rnd(8));
				}
				else if(movement && rnd(30)==0) // Add falling leaf now and then while running on branch
					AddParticle(6, player.x+16, branches[i].y, rnd(16)-8, rnd(8));
				player.dy=0;
			}
	}
	if(player.air>0)
		player.image=37;
	if(player.reload>0)
	{
		if(player.air==0)
			player.image=38;
		else
			player.image=39;
	}
	if(player.hurt>0)
		player.hurt--;

	// Check if player drowned
	if(player.y>364-water_level/16)
	{
		PlaySound(6, 30, 100);
		PlaySound(11, 20, 70);
		death_delay=1;
	}

	// Check if player collided with collectables
	for(int i=0;i<MAX_PARTICLES;i++)
		if(particles[i].exists && (particles[i].type==1 || particles[i].type==2))
		{
			if(player.x<particles[i].x+32
			&& player.x+64>particles[i].x+32
			&& player.y<particles[i].y+32
			&& player.y+64>particles[i].y+32)
			{
				particles[i].exists=false;
				if(particles[i].type==1) // Egg
				{
					PlaySound(7, 10, 100);
					player.score+=400;
					AddParticle(5, particles[i].x+16, particles[i].y+32, 0, 0); // Add score thingy
				}
				else // Heart
				{
					PlaySound(8, 40, 100);
					if(player.health<4)
						player.health++;
				}
			}
		}
	
	// Check if player collided with enemies
	if(player.hurt==0)
		for(int i=0;i<MAX_ENEMIES;i++)
			if(enemies[i].exists)
			{
				if(player.x+32<enemies[i].x+enemies[i].size
				&& player.x+32>enemies[i].x
				&& player.y+32<enemies[i].y+enemies[i].size
				&& player.y+32>enemies[i].y)
				{
					PlaySound(2, 40, 100);
					player.health--;
					player.hurt=20*4-1;
					player.dy=-32;
					if(player.x+32>enemies[i].x+enemies[i].size/2)
						player.dx=64;
					else
						player.dx=-64;
					if(player.health<=0)
					{
						death_delay=1;
						PlaySound(11, 20, 70);
					}
				}
			}
	
	// Update scroll
	scroll_y-=scroll_offset; // Restore scroll
	if(GetAsyncKeyState(VK_DOWN)&KF_UP)
	{
		scroll_offset_delay++;
		if(scroll_offset_delay>20)
			scroll_offset+=4;
	}
	else
	{
		scroll_offset_delay=0;
		scroll_offset-=4;
	}
	if(scroll_offset<=0)
		scroll_offset=0;
	if(scroll_offset>200)
		scroll_offset=200;
	if(player.x-320-84>scroll_x)
		scroll_x=player.x-320-84;
	if(player.x-320+116<scroll_x)
		scroll_x=player.x-320+116;
	if(player.y-240-50>scroll_y)
		scroll_y=player.y-240-50;
	if(player.y-240+100<scroll_y)
		scroll_y=player.y-240+100;
	scroll_y+=scroll_offset;

	// Update harpoon
	if(player.pooning==1)
	{
		player.poon_y-=12;
		// See if it stuck to a branch
		for(int i=0;i<MAX_BRANCHES;i++)
			if(branches[i].exists && player.poon_x>branches[i].x-branches[i].width1*32
								  && player.poon_x<branches[i].x+branches[i].width2*32+32
								  && player.poon_y>branches[i].y
								  && player.poon_y<branches[i].y+15)
			{
				PlaySound(1, 40, 170);
				PlaySound(10, 20, 100);
				player.pooning=2;
				player.dy-=32;
				player.poon_y=branches[i].y+2;
				for(int d=0;d<5;d++) // Add leaves
					AddParticle(6, player.poon_x+rnd(32)-16, branches[i].y, rnd(16)-8, rnd(8));
				break;
			}
		if(player.y-player.poon_y>500) // Shot too far
			player.pooning=3;
	}
	if(player.pooning==2)
	{
		player.air=20;
		player.dx+=(player.poon_x-player.x)/16;
		player.dy-=4;
		if(player.dy>0) // Extra pull when falling
			player.dy-=2;
		if(player.dy<-128)
			player.dy=-128;
		if(player.y+32<player.poon_y)
		{
			player.pooning=0;
			player.dy-=player.dy/3;
			if(player.dy<-96)
				player.dy=-96;
		}
	}
	if(player.pooning>=3) // Let harpoon fly off screen
	{
		player.poon_y-=12;
		player.pooning++;
		if(player.pooning>60)
			player.pooning=0;
	}

	// Update plant
	int dx, dy;
	switch(plant.dir)
	{
	case 0:
		dx=0;
		dy=-1;
		break;
	case 1:
		dx=0;
		dy=1;
		break;
	case 2:
		dx=-1;
		dy=0;
		break;
	case 3:
		dx=1;
		dy=0;
		break;
	}
	plant.x+=dx;
	plant.y+=dy;
	plant.timer--;
	if(plant.timer==0) // Grew one tile
	{
		plant.timer=64;
		Tile *curtile=GetNewTile();
		curtile->image=18+plant.dir/2;
		curtile->layer=0;
		curtile->x=plant.x;
		curtile->y=plant.y;
		curtile->size=64;
		curtile->color=1;
		curtile->timer=2048;
		curtile->occupied=false;
		plant.growth--;
		if(plant.y>150 && plant.dir==1) // Heading into the water, turn immediately
			plant.growth=0;
		if(plant.growth==0) // Grew one branch, change direction
		{
			int pdir=plant.dir;
			plant.dir=(1-plant.dir/2)*2+rnd(1);
			if(plant.y>150 && plant.dir==1) // Prevent plant from growing into the water
				plant.dir=0;
			plant.growth=rnd(5)+2;
			if((pdir==0 && plant.dir==2) || (pdir==3 && plant.dir==1))
				curtile->image=14;
			if((pdir==0 && plant.dir==3) || (pdir==2 && plant.dir==1))
				curtile->image=15;
			if((pdir==1 && plant.dir==2) || (pdir==3 && plant.dir==0))
				curtile->image=16;
			if((pdir==1 && plant.dir==3) || (pdir==2 && plant.dir==0))
				curtile->image=17;
		}
	}

	branch_interval--;
	if(branch_interval==0) // Grow new branch
	{
		branch_interval=50+rnd(50);
		for(int i=0;i<MAX_TILES;i++)
			if(rnd(10)==0 && tiles[i].exists && tiles[i].timer>700 && tiles[i].timer<1900 && !tiles[i].occupied)
			{
				tiles[i].occupied=true;
				for(int j=0;j<MAX_BRANCHES;j++) // Find new branch
					if(!branches[j].exists)
					{
						branches[j].exists=true;
						branches[j].x=tiles[i].x+16;
						branches[j].y=tiles[i].y+16;
						branches[j].width1=0;
						branches[j].width2=0;
						branches[j].growth1=1+rnd(4);
						branches[j].growth2=1+rnd(4);
						branches[j].timer=8;
						branches[j].age=512+rnd(512);
						branches[j].color=1;
						break;
					}
				break;
			}
	}
	
	// Plant aging
	for(int i=0;i<MAX_TILES;i++)
		if(tiles[i].exists && tiles[i].timer>0)
		{
			tiles[i].timer--;
			if(tiles[i].timer<512)
				tiles[i].color=17-tiles[i].timer/32;
		}

	// Update branches
	for(int i=0;i<MAX_BRANCHES;i++)
		if(branches[i].exists)
		{
			branches[i].timer--;
			if(branches[i].timer==0)
			{
				branches[i].timer=8;
				if(branches[i].width1<branches[i].growth1)
					branches[i].width1++;
				if(branches[i].width2<branches[i].growth2)
					branches[i].width2++;
			}
			if(branches[i].age<256)
				branches[i].color=17-branches[i].age/16;
			branches[i].age--;
			if(branches[i].age==0)
			{
				branches[i].exists=false;
				// Spawn dead leaf particles
				for(int d=0;d<(branches[i].width1+branches[i].width2+1)*2;d++)
					AddParticle(7, branches[i].x-branches[i].width1*32+d*16, branches[i].y-16, rnd(16)-8, rnd(8));
			}
		}

	// Update particles
	for(int i=0;i<MAX_PARTICLES;i++)
		if(particles[i].exists)
		{
			switch(particles[i].type)
			{
			case 0: // Food
				particles[i].timer++;
				if(particles[i].timer>15)
					particles[i].timer=0;
				particles[i].x+=particles[i].dx;
				particles[i].y+=particles[i].dy/5;
				particles[i].dy+=1;
				if(particles[i].y>scroll_y+640)
					particles[i].exists=false;
				break;
			case 1: // Egg
			case 2: // Heart
				particles[i].y+=particles[i].dy/8;
				particles[i].dy+=1;
				if(particles[i].y>scroll_y+640)
					particles[i].exists=false;
				break;
			case 3: // Debris
				particles[i].x+=particles[i].dx/4;
				particles[i].dx+=rnd(2)-1;
				particles[i].y+=particles[i].dy/5;
				particles[i].dy+=1;
				if(particles[i].y>scroll_y+640)
					particles[i].exists=false;
				break;
			case 4: // Burst
				particles[i].timer++;
				if(particles[i].timer>8)
					particles[i].exists=false;
				break;
			case 5: // Score
				particles[i].y-=1;
				particles[i].timer++;
				if(particles[i].timer>50)
					particles[i].exists=false;
				break;
			case 6: // Leaf
			case 7: // Dead leaf
				particles[i].x+=particles[i].dx/4;
				particles[i].dx+=rnd(2)-1;
				particles[i].y+=particles[i].dy/5;
				particles[i].dy+=1;
				if(particles[i].dy>16)
					particles[i].dy=16;
				if(particles[i].y>scroll_y+640)
					particles[i].exists=false;
				break;
			}
		}
	
	// Update enemies
	enemy_spawn--;
	if(enemy_spawn<=0) // Spawn new enemy
	{
		enemy_rate-=2;
		enemy_spawn=enemy_rate+rnd(enemy_rate*2);
		for(int i=0;i<MAX_ENEMIES;i++)
			if(!enemies[i].exists)
			{
				enemies[i].exists=true;
				enemies[i].type=0;
				enemies[i].dx=0;
				enemies[i].dy=0;
				enemies[i].dir=0;
				enemies[i].timer=0;
				enemies[i].size=64;
				enemies[i].hurt=0;
				int entry=rnd(2);
				switch(entry)
				{
					case 0: // Left screen edge
						enemies[i].x=scroll_x-64;
						enemies[i].y=scroll_y+rnd(480);
						break;
					case 1: // Right screen edge
						enemies[i].x=scroll_x+640;
						enemies[i].y=scroll_y+rnd(480);
						break;
					case 2: // Top screen edge
						enemies[i].x=scroll_x+rnd(640);
						enemies[i].y=scroll_y-64;
						break;
				}
				break;
			}
	}
	for(int i=0;i<MAX_ENEMIES;i++)
		if(enemies[i].exists)
		{
			if(enemies[i].hurt>0)
				enemies[i].hurt--;
			switch(enemies[i].type)
			{
			case 0: // Pig
				enemies[i].timer++;
				if(enemies[i].timer>31)
					enemies[i].timer=0;
				if(abs(player.x-enemies[i].x)>250)
					enemies[i].dx+=(player.x-enemies[i].x)/128;
				enemies[i].dx+=rnd(4)-2;
				if(abs(player.y-enemies[i].y)>250)
					enemies[i].dy+=(player.y-enemies[i].y)/128;
				enemies[i].dy+=rnd(4)-2;
				if(enemies[i].dx>0) enemies[i].dx++;
				if(enemies[i].dx<0) enemies[i].dx--;
				if(enemies[i].dx>8*4) enemies[i].dx=8*4;
				if(enemies[i].dx<-8*4) enemies[i].dx=-8*4;
				if(enemies[i].dy>8*4) enemies[i].dy=8*4;
				if(enemies[i].dy<-8*4) enemies[i].dy=-8*4;
				if(enemies[i].dx>16) enemies[i].dir=0;
				if(enemies[i].dx<-16) enemies[i].dir=1;
				enemies[i].x+=enemies[i].dx/16;
				enemies[i].y+=enemies[i].dy/16;
				break;
			}
			// Check for collision with food
			for(int j=0;j<MAX_PARTICLES;j++)
				if(particles[j].exists && particles[j].type==0)
				{
					if(enemies[i].x<particles[j].x+32
					&& enemies[i].x+enemies[i].size>particles[j].x+16
					&& enemies[i].y<particles[j].y+32
					&& enemies[i].y+enemies[i].size>particles[j].y+16)
					{
						player.score+=25;
						particles[j].exists=false;
						enemies[i].size+=12; // Enemy grows
						enemies[i].x-=6;
						enemies[i].y-=6;
						enemies[i].hurt=4;
						if(enemies[i].size>128) // Pop
						{
							PlaySound(5, 50, 100);
							player.score+=150;
							enemies[i].exists=false;
							// Spawn debris
							for(int d=0;d<20;d++)
								AddParticle(3, enemies[i].x+enemies[i].size/2+rnd(64)-32, enemies[i].y+enemies[i].size/2+rnd(64)-32, rnd(16)-8, -rnd(32));
							AddParticle(4, enemies[i].x+enemies[i].size/2-48, enemies[i].y+enemies[i].size/2-48, 0, 0);
							// Spawn egg or heart
							AddParticle(1+rnd(20)/20, enemies[i].x+enemies[i].size/2-16, enemies[i].y+enemies[i].size/2-16, 0, -4*8);
						}
						else
							PlaySound(4, 20, 100);
						break;
					}
				}
		}

	if(water_level<96*16) water_level++;
	water_scroll++;
	if(water_scroll>=64)
		water_scroll=0;
	
	return true;
}

#include "toolcode.h"

void Render()
{
//	glClear(GL_COLOR_BUFFER_BIT);
	glLoadIdentity();
	glDepthMask(0);
	glDisable(GL_DEPTH_TEST);
	glColor3f(1,1,1);
	glEnable(GL_TEXTURE_2D);

	if(menu)
	{
		glDisable(GL_TEXTURE_2D);
		glBegin(GL_QUADS);
		glColor3f(0.2f, 0.3f, 0.5f);
		glVertex3i(640,0,0);
		glVertex3i(0,0,0);
		glColor3f(0.0f, 0.0f, 0.0f);
		glVertex3i(0,480,0);
		glVertex3i(640,480,0);
		glEnd();
		glColor3f(1,1,1);
		glEnable(GL_TEXTURE_2D);

		// Title
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glBindTexture(GL_TEXTURE_2D, textures[115].handle);
		glBegin(GL_QUADS);
		glColor3f(0.0f, 0.3f, 0.0f);
		glTexCoord2f(0,1);
		glVertex3i(64,64,0);
		glTexCoord2f(1,1);
		glVertex3i(64+512,64,0);
		glColor3f(1.0f, 1.0f, 0.0f);
		glTexCoord2f(1,0);
		glVertex3i(64+512,64+128,0);
		glTexCoord2f(0,0);
		glVertex3i(64,64+128,0);
		glEnd();
		glColor3f(1,1,1);

		// Jackie
		DrawSpriteScroll(320-64, 290, 31+(((menu_scroller+1000)/6)%6), 128, 128, false);

		// Scroller
		char string[256];
		int index=0;
		for(int i=menu_scroller/8;i<menu_scroller/8+21*8;i++)
		{
			if(i>=0)
				string[index++]=menu_text[i];
			else
				string[index++]=' ';
		}
		string[index]='\0';
		DrawString(-(menu_scroller%8)*4, 425+4, string, 32, 25, 25);
		DrawString(-(menu_scroller%8)*4, 425, string, 32, 23, 24);

		glDisable(GL_BLEND);
		glFlush();
		return;
	}
	
	// Draw background
	glBindTexture(GL_TEXTURE_2D, textures[110].handle);
	float texscroll=-(float)scroll_y/(480*16);
	glBegin(GL_QUADS);
	glTexCoord2f(0,texscroll+0.3f);
	glVertex3i(640,0,0);
	glTexCoord2f(0,texscroll+0.3f);
	glVertex3i(0,0,0);
	glTexCoord2f(0,texscroll);
	glVertex3i(0,480,0);
	glTexCoord2f(0,texscroll);
	glVertex3i(640,480,0);
	glEnd();
	
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// Draw stalk
	for(int curlayer=0;curlayer<4;curlayer++)
		for(int i=0;i<MAX_TILES;i++)
			if(tiles[i].exists && tiles[i].layer==curlayer)
			{
				glColor3f(colors[tiles[i].color].r, colors[tiles[i].color].g, colors[tiles[i].color].b);
				DrawSpriteScroll(tiles[i].x, tiles[i].y, tiles[i].image, tiles[i].size, tiles[i].size, false);
			}

	// Draw harpoon
	glColor3f(1,1,1);
	if(player.pooning>0)
	{
		int poon_end=0;
		if(player.pooning>2)
			poon_end+=(player.pooning-2)*12;
		DrawSpriteScroll(player.poon_x-16, player.poon_y+4, 40, 32, 32, false);
		float px=(float)player.poon_x;
		float dx=(float)(player.x+32-player.poon_x)/(player.y-player.poon_y);
		int dy=(int)(13-dx/2);
		if(dy<2) dy=2;
		if(player.y-player.poon_y>32)
			for(int y=player.poon_y+32;y<player.y+16-poon_end;y+=dy)
			{
				px+=dx*dy;
				DrawSpriteScroll((int)px-8, y, 41, 16, 16, false);
			}
	}

	// Draw branches
	for(int i=0;i<MAX_BRANCHES;i++)
		if(branches[i].exists)
		{
			glColor3f(colors[branches[i].color].r, colors[branches[i].color].g, colors[branches[i].color].b);
			DrawSpriteScroll(branches[i].x, branches[i].y, 1, 32, 32, false);
			for(int x=0;x<branches[i].width1+1;x++)
				if(x==0)
					DrawSpriteScroll(branches[i].x-branches[i].width1*32+x*32, branches[i].y, 3, 32, 32, false);
				else
					DrawSpriteScroll(branches[i].x-branches[i].width1*32+x*32, branches[i].y, 2, 32, 32, false);
			for(int x=0;x<branches[i].width2;x++)
				if(x==branches[i].width2-1)
					DrawSpriteScroll(branches[i].x+32+x*32, branches[i].y, 4, 32, 32, false);
				else
					DrawSpriteScroll(branches[i].x+32+x*32, branches[i].y, 2, 32, 32, false);
		}
	glColor3f(1,1,1);

	// Draw particles
	for(int i=0;i<MAX_PARTICLES;i++)
		if(particles[i].exists)
			switch(particles[i].type)
			{
			case 0: // Food
				DrawSpriteScroll(particles[i].x, particles[i].y, 60+particles[i].timer/4, 32, 32, false);
				break;
			case 1: // Egg
				DrawSpriteScroll(particles[i].x, particles[i].y, 49, 64, 64, false);
				break;
			case 2: // Heart
				glColor3f(1.0f, 0.3f, 0.1f);
				DrawSpriteScroll(particles[i].x, particles[i].y, 55, 32, 32, false);
				glColor3f(1,1,1);
				break;
			case 3: // Debris
				DrawSpriteScroll(particles[i].x, particles[i].y, 56, 32, 32, false);
				break;
			case 4: // Burst
				if(particles[i].timer&2)
					DrawSpriteScroll(particles[i].x, particles[i].y, 57, 128, 128, false);
				break;
			case 5: // Score
				DrawSpriteScroll(particles[i].x, particles[i].y, 58, 32, 16, false);
				break;
			case 6: // Leaf
				glColor3f(colors[1].r, colors[1].g, colors[1].b);
				DrawSpriteScroll(particles[i].x, particles[i].y, 59, 32, 32, particles[i].dx>0);
				glColor3f(1,1,1);
				break;
			case 7: // Dead leaf
				glColor3f(colors[17].r, colors[17].g, colors[17].b);
				DrawSpriteScroll(particles[i].x, particles[i].y, 59, 32, 32, particles[i].dx>0);
				glColor3f(1,1,1);
				break;
			}

	// Draw enemies
	for(int i=0;i<MAX_ENEMIES;i++)
		if(enemies[i].exists)
			switch(enemies[i].type)
			{
			case 0: // Pig
				if(enemies[i].hurt==0)
					DrawSpriteScroll(enemies[i].x, enemies[i].y, 45+enemies[i].timer/8, enemies[i].size, enemies[i].size, enemies[i].dir==1);
				else
					DrawSpriteScroll(enemies[i].x, enemies[i].y, 45+enemies[i].timer/8+128, enemies[i].size, enemies[i].size, enemies[i].dir==1);
				break;
			}

	// Draw player
	if((player.hurt&4)==0)
		DrawSpriteScroll(player.x, player.y, player.image, 64, 64, player.dir==1);
	else
		DrawSpriteScroll(player.x, player.y, player.image+128, 64, 64, player.dir==1);
	
	// Draw flower
	DrawSpriteScroll(plant.x-32, plant.y-32, 20, 128, 128, false);

	// Draw water
	scroll_x+=water_scroll;
	for(int x=scroll_x/64*64-64;x<scroll_x/64*64+640+64;x+=64)
	{
		DrawSpriteScroll(x, 364-water_level/16, 10, 64, 64, false);
		for(int y=1;y<7;y++)
			DrawSpriteScroll(x, 364+y*64-water_level/16, 11, 64, 64, false);
	}
	scroll_x-=water_scroll;

	// HUD	
	char string[256];
	sprintf(string, "SCORE %.8i", player.score);
	DrawString(16+4, 425+4, string, 32, 20, 22);
	DrawString(16, 425, string, 32, 0, 20);
	for(int i=0;i<4;i++)
	{
		if(i<player.health)
			glColor3f(1.0f, 0.3f, 0.1f);
		else
			glColor3f(0.2f, 0.3f, 0.4f);
		DrawSprite(490+i*32, 425, 55, 32, 32, false);
	}

	if(death_delay>0) // Draw red tint
	{
		glDisable(GL_TEXTURE_2D);
		glBlendFunc(GL_ONE, GL_DST_COLOR);
		glColor3f(0.5f, 0.0f, 0.0f);
		glBegin(GL_QUADS);
		glVertex3i(640,0,0);
		glVertex3i(0,0,0);
		glVertex3i(0,480,0);
		glVertex3i(640,480,0);
		glEnd();
		glEnable(GL_TEXTURE_2D);
	}

	glDisable(GL_BLEND);
	glFlush();
}

bool InitGame(HWND hWnd)
{
	srand(time(NULL));

	BASS_Init(1, 44100, 0, hWnd, NULL);
	BASS_Start();
	music_sample=BASS_SampleLoad(FALSE, "data\\sfx\\music.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	music_channel=BASS_SampleGetChannel(music_sample, false);
	BASS_ChannelSetAttributes(music_channel, 22050, 20, 0);
	BASS_ChannelSetFlags(music_channel, BASS_SAMPLE_LOOP);
	for(int i=0;i<32;i++)
		sound_channel[i]=NULL;
	sound_sample[0]=BASS_SampleLoad(FALSE, "data\\sfx\\jump.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[1]=BASS_SampleLoad(FALSE, "data\\sfx\\land.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[2]=BASS_SampleLoad(FALSE, "data\\sfx\\hurt.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[3]=BASS_SampleLoad(FALSE, "data\\sfx\\throw.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[4]=BASS_SampleLoad(FALSE, "data\\sfx\\grow.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[5]=BASS_SampleLoad(FALSE, "data\\sfx\\pop.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[6]=BASS_SampleLoad(FALSE, "data\\sfx\\splash.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[7]=BASS_SampleLoad(FALSE, "data\\sfx\\egg.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[8]=BASS_SampleLoad(FALSE, "data\\sfx\\heart.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[9]=BASS_SampleLoad(FALSE, "data\\sfx\\grapple.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[10]=BASS_SampleLoad(FALSE, "data\\sfx\\swing.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
	sound_sample[11]=BASS_SampleLoad(FALSE, "data\\sfx\\death.wav", 0, 0, 1, BASS_SAMPLE_OVER_POS);
		
	
	LoadTGATexture(1, "data\\branch1.tga");
	LoadTGATexture(2, "data\\branch2.tga");
	LoadTGATexture(3, "data\\branch3.tga");
	LoadTGATexture(4, "data\\branch4.tga");
	LoadTGATexture(5, "data\\ground1.tga");
	LoadTGATexture(6, "data\\ground2.tga");
	LoadTGATexture(7, "data\\ground3.tga");
	LoadTGATexture(8, "data\\black.tga");
	LoadTGATexture(10, "data\\water1.tga");
	LoadTGATexture(11, "data\\water2.tga");

	LoadTGATexture(14, "data\\corner1.tga");
	LoadTGATexture(15, "data\\corner2.tga");
	LoadTGATexture(16, "data\\corner3.tga");
	LoadTGATexture(17, "data\\corner4.tga");
	LoadTGATexture(18, "data\\stalk1.tga");
	LoadTGATexture(19, "data\\stalk2.tga");
	LoadTGATexture(20, "data\\flower.tga");

	LoadTGATexture(30, "data\\player1.tga");
	LoadTGATexture(31, "data\\player2.tga");
	LoadTGATexture(32, "data\\player3.tga");
	LoadTGATexture(33, "data\\player4.tga");
	LoadTGATexture(34, "data\\player42.tga");
	LoadTGATexture(35, "data\\player2.tga");
	LoadTGATexture(36, "data\\player5.tga");
	LoadTGATexture(37, "data\\player6.tga");
	LoadTGATexture(38, "data\\player7.tga");
	LoadTGATexture(39, "data\\player8.tga");

	LoadTGATexture(40, "data\\poon1.tga");
	LoadTGATexture(41, "data\\poon2.tga");

	LoadTGATexture(45, "data\\pig1.tga");
	LoadTGATexture(46, "data\\pig2.tga");
	LoadTGATexture(47, "data\\pig3.tga");
	LoadTGATexture(48, "data\\pig2.tga");
	LoadTGATexture(49, "data\\egg.tga");

	LoadTGATexture(55, "data\\heart.tga");
	LoadTGATexture(56, "data\\debris1.tga");
	LoadTGATexture(57, "data\\burst.tga");
	LoadTGATexture(58, "data\\score.tga");
	LoadTGATexture(59, "data\\leaf1.tga");

	LoadTGATexture(60, "data\\weapon1.tga");
	LoadTGATexture(61, "data\\weapon2.tga");
	LoadTGATexture(62, "data\\weapon3.tga");
	LoadTGATexture(63, "data\\weapon4.tga");

	LoadTGATexture(100, "data\\font1.tga");
	LoadTGATexture(101, "data\\font2.tga");
	LoadTGATexture(102, "data\\font3.tga");

	LoadTGATexture(110, "data\\bg.tga");

	LoadTGATexture(115, "data\\title.tga");
	
	strcpy(menu_text, "PRESS RETURN TO START GAME...CONTROLS - ARROWS.Z.X...UP PLUS X IS GRAPPLING HOOK...DOWN LOOKS DOWN...DOWN PLUS Z DROPS FROM PLATFORM...THIS IS DRPETTERS ENTRY FOR LUDUM DARE 48H COMPO DEC 2005 - GREETINGS TO ALL FELLOW LUDUMERS...I AM SHORT ON TIME SO THIS SCROLLER WILL NOT BE VERY LONG NOR WITTY...HA DET GOTT OCH LEK INTE MED MATEN......THE DOCTOR WILL NOW SLEEP FOR ONE HALF ETERNITY...");

	// Init colors
	colors[0].r=1.0f; colors[0].g=1.0f; colors[0].b=1.0f;
	colors[1].r=0.4f; colors[1].g=0.7f; colors[1].b=0.35f;
	colors[17].r=0.6f; colors[17].g=0.5f; colors[17].b=0.4f;
	for(int i=2;i<17;i++)
	{
		float f=(float)(i-1)/16;
		colors[i].r=colors[1].r*(1.0f-f)+colors[17].r*f;
		colors[i].g=colors[1].g*(1.0f-f)+colors[17].g*f;
		colors[i].b=colors[1].b*(1.0f-f)+colors[17].b*f;
	}
	colors[20].r=0.7f; colors[20].g=0.3f; colors[20].b=0.5f;
	colors[21].r=0.35f; colors[21].g=0.15f; colors[21].b=0.25f;
	colors[22].r=0.0f; colors[22].g=0.0f; colors[22].b=0.0f;
	colors[23].r=1.0f; colors[23].g=0.0f; colors[23].b=0.0f;
	colors[24].r=1.0f; colors[24].g=1.0f; colors[24].b=0.0f;
	colors[25].r=0.3f; colors[25].g=0.3f; colors[25].b=0.3f;

	ResetGame();
	
	return true;
}

